@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.25.2.1 2003/02/12 10:49:09 bazsi Exp $ */
+ * $Id: io.c,v 1.27 2002/03/21 11:32:35 bazsi Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -231,14 +231,14 @@ int io_iter(struct io_backend *b)
 			if (!fd->super.alive)
 				continue;
 			
-			if (fds[i].revents & POLLHUP) {
+			if (fds[i].revents & (POLLHUP|POLLERR|POLLNVAL)) {
 				if (fd->want_read && fd->read)
 					READ_FD(fd);
 				else if (fd->want_write && fd->write)
 					WRITE_FD(fd);
 				else {
 					verbose("io_iter(): POLLHUP on inactive fd!\n");
-					closekill_fd(fd, CLOSE_EOF);
+					close_fd(fd, CLOSE_EOF);
 				}
 				continue;
 			}
@@ -246,10 +246,12 @@ int io_iter(struct io_backend *b)
 				close_fd(fd, CLOSE_PROTOCOL_FAILURE);
 				continue;
 			}
+			/*
 			if (fds[i].revents & (POLLNVAL | POLLERR)) {
 				close_fd(fd, CLOSE_POLL_FAILED);
 				continue;
 			}
+			*/
 			if (fds[i].revents & POLLOUT)
 				if (fd->want_write && fd->write)
 					WRITE_FD(fd);
@@ -451,8 +453,10 @@ static void read_callback(struct nonblocking_fd *fd)
 static int do_write(struct abstract_write *w, UINT32 length, UINT8 *data)
 {
 	CAST(fd_write, self, w);
+	int saved_errno;
 
 	int res = write(self->fd, data, length);
+	saved_errno = errno;
 
 #if 0
 	if (res>=0)
@@ -476,6 +480,7 @@ static int do_write(struct abstract_write *w, UINT32 length, UINT8 *data)
 	}
 	if (self->fsync)
 		fsync(self->fd);
+	errno = saved_errno;
 	return res;
 }
 
