@@ -18,7 +18,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: pkt_buffer.c,v 1.12 2001/10/11 18:18:58 bazsi Exp $
+ * $Id: pkt_buffer.c,v 1.14 2002/03/21 11:32:35 bazsi Exp $
  *
  ***************************************************************************/
  
@@ -88,36 +88,69 @@ static int do_write_str(struct abstract_write *c, struct ol_string *string)
 static int do_flush(struct abstract_buffer *c, struct abstract_write *w)
 {
 	CAST(pkt_buffer, self, c); 
- 	int res;
-
-	FOR_QUEUE(&self->queue, struct buffer_node *, item) {
-
-		res = A_WRITE(w, item->packet->length, item->packet->data);
+	char buffer[4096];
+	int res;
+	int end;
+	
+	do {
+		end = 0;
+		{
+			FOR_QUEUE(&self->queue, struct buffer_node *, item) {
+				int avail = sizeof(buffer) - end;
+				int move = MIN(item->packet->length, avail);
+				
+				if (avail == 0)
+					break;
+				memcpy(&buffer[end], item->packet->data, move);
+				end += move;
+			
+				if (item->packet->length != move) {
+					struct ol_string *s;
+					
+					s = item->packet;
+					item->packet = c_format("%s", s->length - move, s->data + move);
+					ol_string_free(s);
+				} 
+				else {
+					
+					/* successfully moved */
+					self->queue_size--;
+					ol_queue_remove((struct ol_queue_node *) item);
+					ol_string_free(item->packet);
+					ol_space_free(item);
+				}
+			}
+		}
+		
+		if (end == 0)
+			break;
+			
+		res = A_WRITE(w, end, buffer);
+		
 		if (res >= 0) {
-			if (item->packet->length == res) {
-				self->queue_size--;
-				ol_queue_remove((struct ol_queue_node *) item);
-				ol_string_free(item->packet);
-				ol_space_free(item);
+			if (end == res) {
 				if (self->super.writable)
 					(*self->super.writable) = 1;
 			}
 			else if (res != 0) {
-				struct ol_string *s;
-				s = item->packet;
-				item->packet = c_format("%s", s->length - res, s->data + res);
-				ol_string_free(s);
-				break;
-			}
-			else {
+				/* this is slow, because of another memory move
+				 * but this is run rarely anyway */
+				struct buffer_node *item;
+				
+				NEW_SPACE(item);
+				item->packet = c_format("%s", end - res, buffer + res);
+				ol_queue_add_head(&self->queue, &item->header);
+				if (++self->queue_size == self->queue_max && self->super.writable)
+					(*self->super.writable) = 0;
 				break;
 			}
 		}
 		else {
-			verbose("pkt_buffer::do_flush(): Error flushing data\n");
+			werror("pkt_buffer::do_flush(): Error flushing data");
 			return ST_DIE;
 		}
 	}
+	while (1);
 	return ST_OK | ST_GOON;
 }
 
